import { usePress } from "@react-aria/interactions";
import { useFocusable } from "@react-aria/focus";
import { useSelectableCollection } from "@react-aria/selection";
import { useCollator, useLocale } from "@react-aria/i18n";
import { useMemo } from "react";
import { filterDOMProps, mergeProps, useId } from "@react-aria/utils";
import _babelRuntimeHelpersEsmExtends from "@babel/runtime/helpers/esm/extends";
import { useGridCell as useTableCell, useGridRowGroup as useTableRowGroup, GridKeyboardDelegate, useGridRow } from "@react-aria/grid";
export { useTableCell, useTableRowGroup };

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
const $fd8271485d8e52f870169aca977a9c1$export$gridIds = new WeakMap();

function $fd8271485d8e52f870169aca977a9c1$var$normalizeKey(key) {
  if (typeof key === 'string') {
    return key.replace(/\s*/g, '');
  }

  return '' + key;
}

function $fd8271485d8e52f870169aca977a9c1$export$getColumnHeaderId(state, columnKey) {
  let gridId = $fd8271485d8e52f870169aca977a9c1$export$gridIds.get(state);

  if (!gridId) {
    throw new Error('Unknown grid');
  }

  return gridId + "-" + $fd8271485d8e52f870169aca977a9c1$var$normalizeKey(columnKey);
}

function $fd8271485d8e52f870169aca977a9c1$export$getCellId(state, rowKey, columnKey) {
  let gridId = $fd8271485d8e52f870169aca977a9c1$export$gridIds.get(state);

  if (!gridId) {
    throw new Error('Unknown grid');
  }

  return gridId + "-" + $fd8271485d8e52f870169aca977a9c1$var$normalizeKey(rowKey) + "-" + $fd8271485d8e52f870169aca977a9c1$var$normalizeKey(columnKey);
}

function $fd8271485d8e52f870169aca977a9c1$export$getRowLabelledBy(state, rowKey) {
  // A row is labelled by it's row headers.
  return [...state.collection.rowHeaderColumnKeys].map(columnKey => $fd8271485d8e52f870169aca977a9c1$export$getCellId(state, rowKey, columnKey)).join(' ');
}

class $dabf6db3fde4ddfaf13a80216b120e5f$export$TableKeyboardDelegate extends GridKeyboardDelegate {
  isCell(node) {
    return node.type === 'cell' || node.type === 'rowheader' || node.type === 'column';
  }

  getKeyBelow(key) {
    let startItem = this.collection.getItem(key);

    if (!startItem) {
      return;
    } // If focus was on a column, then focus the first child column if any,
    // or find the corresponding cell in the first row.


    if (startItem.type === 'column') {
      let child = [...startItem.childNodes][0];

      if (child) {
        return child.key;
      }

      let firstItem = this.collection.getItem(this.getFirstKey());
      return [...firstItem.childNodes][startItem.index].key;
    }

    return super.getKeyBelow(key);
  }

  getKeyAbove(key) {
    let startItem = this.collection.getItem(key);

    if (!startItem) {
      return;
    } // If focus was on a column, focus the parent column if any


    if (startItem.type === 'column') {
      let parent = this.collection.getItem(startItem.parentKey);

      if (parent && parent.type === 'column') {
        return parent.key;
      }

      return;
    } // only return above row key if not header row


    let superKey = super.getKeyAbove(key);

    if (superKey && this.collection.getItem(superKey).type !== 'headerrow') {
      return superKey;
    } // If no item was found, and focus was on a cell, then focus the
    // corresponding column header.


    if (this.isCell(startItem)) {
      return this.collection.columns[startItem.index].key;
    } // If focus was on a row, then focus the first column header.


    return this.collection.columns[0].key;
  }

  findNextColumnKey(column) {
    // Search following columns
    let key = this.findNextKey(column.key, item => item.type === 'column');

    if (key != null) {
      return key;
    } // Wrap around to the first column


    let row = this.collection.headerRows[column.level];

    for (let item of row.childNodes) {
      if (item.type === 'column') {
        return item.key;
      }
    }
  }

  findPreviousColumnKey(column) {
    // Search previous columns
    let key = this.findPreviousKey(column.key, item => item.type === 'column');

    if (key != null) {
      return key;
    } // Wrap around to the last column


    let row = this.collection.headerRows[column.level];
    let childNodes = [...row.childNodes];

    for (let i = childNodes.length - 1; i >= 0; i--) {
      let item = childNodes[i];

      if (item.type === 'column') {
        return item.key;
      }
    }
  }

  getKeyRightOf(key) {
    let item = this.collection.getItem(key);

    if (!item) {
      return;
    } // If focus was on a column, then focus the next column


    if (item.type === 'column') {
      return this.direction === 'rtl' ? this.findPreviousColumnKey(item) : this.findNextColumnKey(item);
    }

    return super.getKeyRightOf(key);
  }

  getKeyLeftOf(key) {
    let item = this.collection.getItem(key);

    if (!item) {
      return;
    } // If focus was on a column, then focus the previous column


    if (item.type === 'column') {
      return this.direction === 'rtl' ? this.findNextColumnKey(item) : this.findPreviousColumnKey(item);
    }

    return super.getKeyLeftOf(key);
  }

  getKeyForSearch(search, fromKey) {
    if (!this.collator) {
      return null;
    }

    let collection = this.collection;
    let key = fromKey != null ? fromKey : this.getFirstKey(); // If the starting key is a cell, search from its parent row.

    let startItem = collection.getItem(key);

    if (startItem.type === 'cell') {
      key = startItem.parentKey;
    }

    let hasWrapped = false;

    while (key != null) {
      let item = collection.getItem(key); // Check each of the row header cells in this row for a match

      for (let cell of item.childNodes) {
        let column = collection.columns[cell.index];

        if (collection.rowHeaderColumnKeys.has(column.key) && cell.textValue) {
          let substring = cell.textValue.slice(0, search.length);

          if (this.collator.compare(substring, search) === 0) {
            // If we started on a cell, end on the matching cell. Otherwise, end on the row.
            let fromItem = fromKey != null ? collection.getItem(fromKey) : startItem;
            return fromItem.type === 'cell' ? cell.key : item.key;
          }
        }
      }

      key = this.getKeyBelow(key); // Wrap around when reaching the end of the collection

      if (key == null && !hasWrapped) {
        key = this.getFirstKey();
        hasWrapped = true;
      }
    }

    return null;
  }

}

export function useTable(props, state) {
  let {
    ref,
    isVirtualized,
    keyboardDelegate,
    layout
  } = props;

  if (!props['aria-label'] && !props['aria-labelledby']) {
    console.warn('An aria-label or aria-labelledby prop is required for accessibility.');
  } // By default, a KeyboardDelegate is provided which uses the DOM to query layout information (e.g. for page up/page down).
  // When virtualized, the layout object will be passed in as a prop and override this.


  let collator = useCollator({
    usage: 'search',
    sensitivity: 'base'
  });
  let {
    direction
  } = useLocale();
  let delegate = useMemo(() => keyboardDelegate || new $dabf6db3fde4ddfaf13a80216b120e5f$export$TableKeyboardDelegate({
    collection: state.collection,
    disabledKeys: state.disabledKeys,
    ref,
    direction,
    collator,
    layout
  }), [keyboardDelegate, state.collection, state.disabledKeys, ref, direction, collator, layout]);
  let {
    collectionProps
  } = useSelectableCollection({
    ref,
    selectionManager: state.selectionManager,
    keyboardDelegate: delegate
  });
  let id = useId();
  $fd8271485d8e52f870169aca977a9c1$export$gridIds.set(state, id);
  let domProps = filterDOMProps(props, {
    labelable: true
  });
  let gridProps = mergeProps(domProps, _babelRuntimeHelpersEsmExtends({
    role: 'grid',
    id,
    'aria-multiselectable': state.selectionManager.selectionMode === 'multiple' ? 'true' : undefined
  }, collectionProps));

  if (isVirtualized) {
    gridProps['aria-rowcount'] = state.collection.size + state.collection.headerRows.length;
    gridProps['aria-colcount'] = state.collection.columns.length;
  }

  return {
    gridProps
  };
}
export function useTableColumnHeader(props, state) {
  let {
    node,
    colspan,
    ref,
    isDisabled
  } = props;
  let {
    gridCellProps
  } = useTableCell(props, state);
  let {
    pressProps
  } = usePress({
    isDisabled: !node.props.allowsSorting || isDisabled,

    onPress() {
      state.sort(node.key);
    }

  }); // Needed to pick up the focusable context, enabling things like Tooltips for example

  let {
    focusableProps
  } = useFocusable({}, ref);
  let ariaSort = null;

  if (node.props.allowsSorting) {
    var _state$sortDescriptor;

    ariaSort = ((_state$sortDescriptor = state.sortDescriptor) == null ? void 0 : _state$sortDescriptor.column) === node.key ? state.sortDescriptor.direction : 'none';
  }

  return {
    columnHeaderProps: _babelRuntimeHelpersEsmExtends({}, mergeProps(gridCellProps, pressProps, focusableProps), {
      role: 'columnheader',
      id: $fd8271485d8e52f870169aca977a9c1$export$getColumnHeaderId(state, node.key),
      'aria-colspan': colspan && colspan > 1 ? colspan : null,
      'aria-sort': ariaSort
    })
  };
}
export function useTableRow(props, state) {
  let {
    node
  } = props;
  let {
    rowProps
  } = useGridRow(props, state);
  return {
    rowProps: _babelRuntimeHelpersEsmExtends({}, rowProps, {
      'aria-labelledby': $fd8271485d8e52f870169aca977a9c1$export$getRowLabelledBy(state, node.key)
    })
  };
}
export function useTableRowHeader(props, state) {
  let {
    gridCellProps
  } = useTableCell(props, state);
  let columnKey = props.node.column.key;
  return {
    rowHeaderProps: _babelRuntimeHelpersEsmExtends({}, gridCellProps, {
      role: 'rowheader',
      id: $fd8271485d8e52f870169aca977a9c1$export$getCellId(state, props.node.parentKey, columnKey)
    })
  };
}
export function useTableSelectionCheckbox(props, state) {
  let {
    key,
    isDisabled
  } = props;
  let manager = state.selectionManager;
  let checkboxId = useId();
  let isSelected = state.selectionManager.isSelected(key) && !isDisabled;

  let onChange = () => manager.select(key);

  return {
    checkboxProps: {
      id: checkboxId,
      'aria-label': 'Select',
      'aria-labelledby': checkboxId + " " + $fd8271485d8e52f870169aca977a9c1$export$getRowLabelledBy(state, key),
      isSelected,
      onChange
    }
  };
}
export function useTableSelectAllCheckbox(state) {
  let {
    isEmpty,
    isSelectAll
  } = state.selectionManager;
  return {
    checkboxProps: {
      'aria-label': 'Select All',
      isSelected: isSelectAll,
      isIndeterminate: !isEmpty && !isSelectAll,
      onChange: () => state.selectionManager.toggleSelectAll()
    }
  };
}
//# sourceMappingURL=module.js.map
